@@ -15,8 +15,7 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   see <http://www.gnu.org/licenses/>.
 
    Author: Stef Walter <stef@memberwebs.com>
 */
@@ -25,7 +24,8 @@
 
 #include "gck.h"
 #include "gck-private.h"
-#include "gck-marshal.h"
+
+#include "gck/gck-marshal.h"
 
 #include <glib/gi18n-lib.h>
 
