@@ -171,12 +171,14 @@ EGG_SECURE_DECLARE (parser);
 static GQuark PEM_CERTIFICATE;
 static GQuark PEM_RSA_PRIVATE_KEY;
 static GQuark PEM_DSA_PRIVATE_KEY;
+static GQuark PEM_EC_PRIVATE_KEY;
 static GQuark PEM_ANY_PRIVATE_KEY;
 static GQuark PEM_ENCRYPTED_PRIVATE_KEY;
 static GQuark PEM_PRIVATE_KEY;
 static GQuark PEM_PKCS7;
 static GQuark PEM_PKCS12;
 static GQuark PEM_CERTIFICATE_REQUEST;
+static GQuark PEM_PUBLIC_KEY;
 
 static GQuark ARMOR_PGP_PUBLIC_KEY_BLOCK;
 static GQuark ARMOR_PGP_PRIVATE_KEY_BLOCK;
@@ -195,11 +197,13 @@ init_quarks (void)
 		QUARK (PEM_PRIVATE_KEY, "PRIVATE KEY");
 		QUARK (PEM_RSA_PRIVATE_KEY, "RSA PRIVATE KEY");
 		QUARK (PEM_DSA_PRIVATE_KEY, "DSA PRIVATE KEY");
+		QUARK (PEM_EC_PRIVATE_KEY, "EC PRIVATE KEY");
 		QUARK (PEM_ANY_PRIVATE_KEY, "ANY PRIVATE KEY");
 		QUARK (PEM_ENCRYPTED_PRIVATE_KEY, "ENCRYPTED PRIVATE KEY");
 		QUARK (PEM_PKCS7, "PKCS7");
 		QUARK (PEM_PKCS12, "PKCS12");
 		QUARK (PEM_CERTIFICATE_REQUEST, "CERTIFICATE REQUEST");
+		QUARK (PEM_PUBLIC_KEY, "PUBLIC KEY");
 
 		QUARK (ARMOR_PGP_PRIVATE_KEY_BLOCK, "PGP PRIVATE KEY BLOCK");
 		QUARK (ARMOR_PGP_PUBLIC_KEY_BLOCK, "PGP PUBLIC KEY BLOCK");
@@ -609,6 +613,70 @@ done:
 	pop_parsed (self, parsed);
 	return ret;
 }
+/* -----------------------------------------------------------------------------
+ * EC PRIVATE KEY
+ */
+
+static gint
+parse_der_private_key_ec (GcrParser *self,
+                          GBytes *data)
+{
+	gint ret = GCR_ERROR_UNRECOGNIZED;
+	GNode *asn = NULL;
+	GBytes *value = NULL;
+	GBytes *pub = NULL;
+	GcrParsed *parsed;
+	guint bits;
+	gulong version;
+
+	parsed = push_parsed (self, TRUE);
+
+	asn = egg_asn1x_create_and_decode (pk_asn1_tab, "ECPrivateKey", data);
+	if (!asn)
+		goto done;
+
+	if (!egg_asn1x_get_integer_as_ulong (egg_asn1x_node (asn, "version", NULL), &version))
+		goto done;
+
+	/* We only support simple version */
+	if (version != 1) {
+		g_message ("unsupported version of EC key: %lu", version);
+		goto done;
+	}
+
+	parsing_block (parsed, GCR_FORMAT_DER_PRIVATE_KEY_EC, data);
+	parsing_object (parsed, CKO_PRIVATE_KEY);
+	parsed_ulong_attribute (parsed, CKA_KEY_TYPE, CKK_EC);
+	parsed_boolean_attribute (parsed, CKA_PRIVATE, CK_TRUE);
+	ret = GCR_ERROR_FAILURE;
+
+	if (!parsed_asn1_element (parsed, asn, "parameters", CKA_EC_PARAMS))
+		goto done;
+
+	value = egg_asn1x_get_string_as_usg (egg_asn1x_node (asn, "privateKey", NULL), egg_secure_realloc);
+	if (!value)
+		goto done;
+
+	parsed_attribute_bytes (parsed, CKA_VALUE, value);
+
+	pub = egg_asn1x_get_bits_as_raw (egg_asn1x_node (asn, "publicKey", NULL), &bits);
+	if (pub && bits  == 8 * g_bytes_get_size (pub))
+		parsed_attribute_bytes (parsed, CKA_EC_POINT, pub);
+	parsed_fire (self, parsed);
+	ret = SUCCESS;
+
+done:
+	if (pub)
+		g_bytes_unref (pub);
+	if (value)
+		g_bytes_unref (value);
+	egg_asn1x_destroy (asn);
+	if (ret == GCR_ERROR_FAILURE)
+		g_message ("invalid EC key");
+
+	pop_parsed (self, parsed);
+	return ret;
+}
 
 /* -----------------------------------------------------------------------------
  * PRIVATE KEY
@@ -623,6 +691,8 @@ parse_der_private_key (GcrParser *self,
 	res = parse_der_private_key_rsa (self, data);
 	if (res == GCR_ERROR_UNRECOGNIZED)
 		res = parse_der_private_key_dsa (self, data);
+	if (res == GCR_ERROR_UNRECOGNIZED)
+		res = parse_der_private_key_ec (self, data);
 
 	return res;
 }
@@ -691,6 +761,25 @@ done:
 	return res;
 }
 
+static gint
+handle_subject_public_key_ec (GcrParser *self,
+                              GcrParsed *parsed,
+                              GBytes *key,
+                              GNode *params)
+{
+	GBytes *bytes;
+
+	parsing_object (parsed, CKO_PUBLIC_KEY);
+	parsed_ulong_attribute (parsed, CKA_KEY_TYPE, CKK_EC);
+
+	bytes = egg_asn1x_encode (params, g_realloc);
+	parsed_attribute_bytes (parsed, CKA_EC_PARAMS, bytes);
+	parsed_attribute_bytes (parsed, CKA_EC_POINT, key);
+	g_bytes_unref (bytes);
+
+	return SUCCESS;
+}
+
 static gint
 parse_der_subject_public_key (GcrParser *self,
                               GBytes *data)
@@ -725,6 +814,9 @@ parse_der_subject_public_key (GcrParser *self,
 	else if (oid == GCR_OID_PKIX1_DSA)
 		ret = handle_subject_public_key_dsa (self, parsed, key, params);
 
+	else if (oid == GCR_OID_PKIX1_EC)
+		ret = handle_subject_public_key_ec (self, parsed, key, params);
+
 	else
 		ret = GCR_ERROR_UNRECOGNIZED;
 
@@ -773,6 +865,8 @@ parse_der_pkcs8_plain (GcrParser *self,
 		key_type = CKK_RSA;
 	else if (key_algo == GCR_OID_PKIX1_DSA)
 		key_type = CKK_DSA;
+	else if (key_algo == GCR_OID_PKIX1_EC)
+		key_type = CKK_EC;
 
 	if (key_type == GCK_INVALID) {
   		ret = GCR_ERROR_UNRECOGNIZED;
@@ -801,6 +895,10 @@ done:
 			if (ret == GCR_ERROR_UNRECOGNIZED && params)
 				ret = parse_der_private_key_dsa_parts (self, keydata, params);
 			break;
+		case CKK_EC:
+			ret = parse_der_private_key_ec (self, keydata);
+			break;
+
 		default:
 			g_message ("invalid or unsupported key type in PKCS#8 key");
 			ret = GCR_ERROR_UNRECOGNIZED;
@@ -1723,6 +1821,9 @@ formats_for_armor_type (GQuark armor_type,
 	} else if (armor_type == PEM_DSA_PRIVATE_KEY) {
 		*inner_format = GCR_FORMAT_DER_PRIVATE_KEY_DSA;
 		*outer_format = GCR_FORMAT_PEM_PRIVATE_KEY_DSA;
+	} else if (armor_type == PEM_EC_PRIVATE_KEY) {
+		*inner_format = GCR_FORMAT_DER_PRIVATE_KEY_EC;
+		*outer_format = GCR_FORMAT_PEM_PRIVATE_KEY_EC;
 	} else if (armor_type == PEM_ANY_PRIVATE_KEY) {
 		*inner_format = GCR_FORMAT_DER_PRIVATE_KEY;
 		*outer_format = GCR_FORMAT_PEM_PRIVATE_KEY;
@@ -1744,6 +1845,9 @@ formats_for_armor_type (GQuark armor_type,
 	} else if (armor_type == PEM_PKCS12) {
 		*inner_format = GCR_FORMAT_DER_PKCS12;
 		*outer_format = GCR_FORMAT_PEM_PKCS12;
+	} else if (armor_type == PEM_PUBLIC_KEY) {
+		*inner_format = GCR_FORMAT_DER_SUBJECT_PUBLIC_KEY;
+		*outer_format = GCR_FORMAT_PEM_PUBLIC_KEY;
 	} else if (armor_type == ARMOR_PGP_PRIVATE_KEY_BLOCK) {
 		*inner_format = GCR_FORMAT_OPENPGP_PACKET;
 		*outer_format = GCR_FORMAT_OPENPGP_ARMOR;
@@ -1933,6 +2037,20 @@ parse_pem_private_key_dsa (GcrParser *self,
 	return handle_pem_format (self, GCR_FORMAT_DER_PRIVATE_KEY_DSA, data);
 }
 
+static gint
+parse_pem_private_key_ec (GcrParser *self,
+		          GBytes *data)
+{
+	return handle_pem_format (self, GCR_FORMAT_DER_PRIVATE_KEY_EC, data);
+}
+
+static gint
+parse_pem_public_key (GcrParser *self,
+                      GBytes *data)
+{
+	return handle_pem_format (self, GCR_FORMAT_DER_SUBJECT_PUBLIC_KEY, data);
+}
+
 static gint
 parse_pem_certificate (GcrParser *self,
                        GBytes *data)
@@ -2028,6 +2146,7 @@ parse_openssh_public (GcrParser *self,
  * @GCR_FORMAT_DER_PRIVATE_KEY: DER encoded private key
  * @GCR_FORMAT_DER_PRIVATE_KEY_RSA: DER encoded RSA private key
  * @GCR_FORMAT_DER_PRIVATE_KEY_DSA: DER encoded DSA private key
+ * @GCR_FORMAT_DER_PRIVATE_KEY_EC: DER encoded EC private key
  * @GCR_FORMAT_DER_SUBJECT_PUBLIC_KEY: DER encoded SubjectPublicKeyInfo
  * @GCR_FORMAT_DER_CERTIFICATE_X509: DER encoded X.509 certificate
  * @GCR_FORMAT_DER_PKCS7: DER encoded PKCS\#7 container file which can contain certificates
@@ -2043,12 +2162,14 @@ parse_openssh_public (GcrParser *self,
  * @GCR_FORMAT_PEM_PRIVATE_KEY: An OpenSSL style PEM file with a private key
  * @GCR_FORMAT_PEM_PRIVATE_KEY_RSA: An OpenSSL style PEM file with a private RSA key
  * @GCR_FORMAT_PEM_PRIVATE_KEY_DSA: An OpenSSL style PEM file with a private DSA key
+ * @GCR_FORMAT_PEM_PRIVATE_KEY_EC: An OpenSSL style PEM file with a private EC key
  * @GCR_FORMAT_PEM_CERTIFICATE_X509: An OpenSSL style PEM file with an X.509 certificate
  * @GCR_FORMAT_PEM_PKCS7: An OpenSSL style PEM file containing PKCS\#7
  * @GCR_FORMAT_PEM_PKCS8_PLAIN: Unencrypted OpenSSL style PEM file containing PKCS\#8
  * @GCR_FORMAT_PEM_PKCS8_ENCRYPTED: Encrypted OpenSSL style PEM file containing PKCS\#8
  * @GCR_FORMAT_PEM_PKCS10: An OpenSSL style PEM file containing PKCS\#10
  * @GCR_FORMAT_PEM_PKCS12: An OpenSSL style PEM file containing PKCS\#12
+ * @GCR_FORMAT_PEM_PUBLIC_KEY: An OpenSSL style PEM file containing a SubjectPublicKeyInfo
  * @GCR_FORMAT_DER_SPKAC: DER encoded SPKAC as generated by HTML5 keygen element
  * @GCR_FORMAT_BASE64_SPKAC: OpenSSL style SPKAC data
  *
@@ -2065,6 +2186,7 @@ static const ParserFormat parser_normal[] = {
 	{ GCR_FORMAT_BASE64_SPKAC, parse_base64_spkac },
 	{ GCR_FORMAT_DER_PRIVATE_KEY_RSA, parse_der_private_key_rsa },
 	{ GCR_FORMAT_DER_PRIVATE_KEY_DSA, parse_der_private_key_dsa },
+	{ GCR_FORMAT_DER_PRIVATE_KEY_EC, parse_der_private_key_ec },
 	{ GCR_FORMAT_DER_SUBJECT_PUBLIC_KEY, parse_der_subject_public_key },
 	{ GCR_FORMAT_DER_CERTIFICATE_X509, parse_der_certificate },
 	{ GCR_FORMAT_DER_PKCS7, parse_der_pkcs7 },
@@ -2083,6 +2205,7 @@ static const ParserFormat parser_formats[] = {
 	{ GCR_FORMAT_DER_PRIVATE_KEY, parse_der_private_key },
 	{ GCR_FORMAT_DER_PRIVATE_KEY_RSA, parse_der_private_key_rsa },
 	{ GCR_FORMAT_DER_PRIVATE_KEY_DSA, parse_der_private_key_dsa },
+	{ GCR_FORMAT_DER_PRIVATE_KEY_EC, parse_der_private_key_ec },
 	{ GCR_FORMAT_DER_SUBJECT_PUBLIC_KEY, parse_der_subject_public_key },
 	{ GCR_FORMAT_DER_CERTIFICATE_X509, parse_der_certificate },
 	{ GCR_FORMAT_DER_PKCS7, parse_der_pkcs7 },
@@ -2105,6 +2228,8 @@ static const ParserFormat parser_formats[] = {
 	{ GCR_FORMAT_PEM_PKCS8_ENCRYPTED, parse_pem_pkcs8_encrypted },
 	{ GCR_FORMAT_PEM_PKCS12, parse_pem_pkcs12 },
 	{ GCR_FORMAT_PEM_PKCS10, parse_pem_pkcs10 },
+	{ GCR_FORMAT_PEM_PRIVATE_KEY_EC, parse_pem_private_key_ec },
+	{ GCR_FORMAT_PEM_PUBLIC_KEY, parse_pem_public_key },
 };
 
 static int
@@ -2218,6 +2343,9 @@ gcr_parser_finalize (GObject *obj)
 	g_ptr_array_free (self->pv->passwords, TRUE);
 	self->pv->passwords = NULL;
 
+	g_free (self->pv->filename);
+	self->pv->filename = NULL;
+
 	G_OBJECT_CLASS (gcr_parser_parent_class)->finalize (obj);
 }
 
@@ -3214,6 +3342,7 @@ gcr_parser_parse_stream (GcrParser *self, GInputStream *input, GCancellable *can
                          GError **error)
 {
 	GcrParsing *parsing;
+	gboolean result;
 
 	g_return_val_if_fail (GCR_IS_PARSER (self), FALSE);
 	g_return_val_if_fail (G_IS_INPUT_STREAM (input), FALSE);
@@ -3225,7 +3354,10 @@ gcr_parser_parse_stream (GcrParser *self, GInputStream *input, GCancellable *can
 	next_state (parsing, state_read_buffer);
 	g_assert (parsing->complete);
 
-	return gcr_parser_parse_stream_finish (self, G_ASYNC_RESULT (parsing), error);
+	result = gcr_parser_parse_stream_finish (self, G_ASYNC_RESULT (parsing), error);
+	g_object_unref (parsing);
+
+	return result;
 }
 
 /**
