@@ -19,8 +19,6 @@
 
 #include "config.h"
 
-#define DEBUG_FLAG GCR_DEBUG_SECRET_EXCHANGE
-#include "gcr-debug.h"
 #include "gcr-secret-exchange.h"
 
 #include "egg/egg-dh.h"
@@ -156,12 +154,12 @@ gcr_secret_exchange_set_property (GObject *obj,
 	case PROP_PROTOCOL:
 		protocol = g_value_get_string (value);
 		if (protocol == NULL) {
-			_gcr_debug ("automatically selecting secret exchange protocol");
+			g_debug ("automatically selecting secret exchange protocol");
 
 		} else {
 			if (g_str_equal (protocol, GCR_SECRET_EXCHANGE_PROTOCOL_1)) {
-				_gcr_debug ("explicitly using secret exchange protocol: %s",
-				            GCR_SECRET_EXCHANGE_PROTOCOL_1);
+				g_debug ("explicitly using secret exchange protocol: %s",
+				         GCR_SECRET_EXCHANGE_PROTOCOL_1);
 				self->pv->explicit_protocol = TRUE;
 			} else {
 				g_warning ("the GcrSecretExchange protocol %s is unsupported defaulting to %s",
@@ -300,11 +298,9 @@ gcr_secret_exchange_begin (GcrSecretExchange *self)
 
 	g_strchug (result);
 
-	if (_gcr_debugging) {
-		gchar *string = g_strescape (result, "");
-		_gcr_debug ("beginning the secret exchange: %s", string);
-		g_free (string);
-	}
+	gchar *string = g_strescape (result, "");
+	g_debug ("beginning the secret exchange: %s", string);
+	g_free (string);
 
 	if (!g_str_has_prefix (result, SECRET_EXCHANGE_PROTOCOL_1_PREFIX))
 		g_warning ("the prepared data does not have the correct protocol prefix");
@@ -326,7 +322,7 @@ derive_key (GcrSecretExchange *self,
 	klass = GCR_SECRET_EXCHANGE_GET_CLASS (self);
 	g_return_val_if_fail (klass->derive_transport_key, FALSE);
 
-	_gcr_debug ("deriving shared transport key");
+	g_debug ("deriving shared transport key");
 
 	peer = key_file_get_base64 (input, GCR_SECRET_EXCHANGE_PROTOCOL_1, "public", &n_peer);
 	if (peer == NULL) {
@@ -416,11 +412,9 @@ gcr_secret_exchange_receive (GcrSecretExchange *self,
 	g_return_val_if_fail (klass->generate_exchange_key, FALSE);
 	g_return_val_if_fail (klass->derive_transport_key, FALSE);
 
-	if (_gcr_debugging) {
-		gchar *string = g_strescape (exchange, "");
-		_gcr_debug ("receiving secret exchange: %s", string);
-		g_free (string);
-	}
+	gchar *string = g_strescape (exchange, "");
+	g_debug ("receiving secret exchange: %s", string);
+	g_free (string);
 
 	/* Parse the input */
 	input = g_key_file_new ();
@@ -562,11 +556,9 @@ gcr_secret_exchange_send (GcrSecretExchange *self,
 
 	g_strchug (result);
 
-	if (_gcr_debugging) {
-		gchar *string = g_strescape (result, "");
-		_gcr_debug ("sending the secret exchange: %s", string);
-		g_free (string);
-	}
+	gchar *string = g_strescape (result, "");
+	g_debug ("sending the secret exchange: %s", string);
+	g_free (string);
 
 	if (!g_str_has_prefix (result, SECRET_EXCHANGE_PROTOCOL_1_PREFIX))
 		g_warning ("the prepared data does not have the correct protocol prefix: %s", result);
@@ -652,7 +644,7 @@ gcr_secret_exchange_default_generate_exchange_key (GcrSecretExchange *exchange,
 {
 	GcrSecretExchangeDefault *data = exchange->pv->default_exchange;
 
-	_gcr_debug ("generating public key");
+	g_debug ("generating public key");
 
 	if (data == NULL) {
 		data = g_new0 (GcrSecretExchangeDefault, 1);
@@ -688,14 +680,14 @@ gcr_secret_exchange_default_derive_transport_key (GcrSecretExchange *exchange,
 	gsize n_ikm;
 	gcry_mpi_t mpi;
 
-	_gcr_debug ("deriving transport key");
+	g_debug ("deriving transport key");
 
 	g_return_val_if_fail (data != NULL, FALSE);
 	g_return_val_if_fail (data->priv != NULL, FALSE);
 
 	mpi = mpi_from_data (peer, n_peer);
 	if (mpi == NULL) {
-		_gcr_debug ("invalid peer mpi");
+		g_debug ("invalid peer mpi");
 		return FALSE;
 	}
 
@@ -737,7 +729,7 @@ gcr_secret_exchange_default_encrypt_transport_data (GcrSecretExchange *exchange,
 	g_return_val_if_fail (data != NULL, FALSE);
 	g_return_val_if_fail (data->key != NULL, FALSE);
 
-	_gcr_debug ("encrypting data");
+	g_debug ("encrypting data");
 
 	gcry = gcry_cipher_open (&cih, EXCHANGE_1_CIPHER_ALGO, EXCHANGE_1_CIPHER_MODE, 0);
 	if (gcry != 0) {
@@ -802,7 +794,7 @@ gcr_secret_exchange_default_decrypt_transport_data (GcrSecretExchange *exchange,
 	g_return_val_if_fail (data != NULL, FALSE);
 	g_return_val_if_fail (data->key != NULL, FALSE);
 
-	_gcr_debug ("decrypting data");
+	g_debug ("decrypting data");
 
 	if (iv == NULL || n_iv != EXCHANGE_1_IV_LENGTH) {
 		g_message ("secret-exchange: invalid or missing iv");
